/* LP- FilterSession
 * STRASSER Michel - 20040211		init
 * */

#include <Python.h>
#include "LPFilterSession.h"
#include <glib.h>
#include <glib-object.h>
#include <stdio.h>
#include "LPDescReader.h"

static GObjectClass *lpFilterSession_parentClass;

static void
impl_LPFilterSession_finalize(GObject *object)
{
	LPFilterSession *self=(LPFilterSession*)object;

	g_free(self->dir);
	g_free(self->filterName);

	if(self->module) {
		Py_DECREF(self->module);
	}

	lpFilterSession_parentClass->finalize(object);
}


static GList* 	
impl_LPFilterSession_getInterfacesList		(LioubovnikFilterSession* super,
						 GObject** exception)
{
	LPFilterSession* self=(LPFilterSession*)super;
	GList* retval=NULL;
	PyObject *pArgs, *pValue;
	PyObject	*portsDirP, *workingDirP;
			
	portsDirP=PyString_FromString(self->portsDir);
	workingDirP=PyString_FromString(self->dir);
	
	pArgs = PyTuple_New(2);
				
	PyTuple_SetItem(pArgs, 0, portsDirP);	
	PyTuple_SetItem(pArgs, 1, workingDirP);	

	pValue = PyObject_CallObject(self->interfacesFunc, pArgs);
	Py_DECREF(pArgs);
	/*
	Py_DECREF(portsDirP);
	Py_DECREF(workingDirP);
	*/
	if (pValue != NULL) {
		int j;
		int n;

		if(!PySequence_Check(pValue)) {
			g_critical("'interfaces' func doesn't return a list of strings");
			return NULL;
		}

		n=PySequence_Size(pValue);
		for(j=0 ; j<n ; j++) {
			PyObject* item;
			char* str;
			item=PySequence_GetItem(pValue, j);

			if(!PyString_Check(item)) {
				g_critical("'interfaces' func doesn't return a list of strings (item %d)", j);
				return NULL;
			}

			str=PyString_AsString(item);

			retval=g_list_append(retval, g_strdup(str));
			
			Py_DECREF(item);
		}
		Py_DECREF(pValue);
	} else {
		PyErr_Print();
		g_critical("Call failed");
		return NULL;
	}

	return retval;
}

static void
impl_LPFilterSession_applyFilter		(LioubovnikFilterSession* super,
						 IG* filledInterfaces,
				 		 GObject** exception)
{
	LPFilterSession* self=(LPFilterSession*)super;
	PyObject	*portsDirP, *workingDirP;
	PyObject	*igP, *pArgs, *pValue;

	igP=PyCObject_FromVoidPtrAndDesc(filledInterfaces, "LibIGenObject", NULL);
	portsDirP=PyString_FromString(self->portsDir);
	workingDirP=PyString_FromString(self->dir);

	pArgs = PyTuple_New(3);
	PyTuple_SetItem(pArgs, 0, portsDirP);	
	PyTuple_SetItem(pArgs, 1, workingDirP);	
	PyTuple_SetItem(pArgs, 2, igP);	
	pValue = PyObject_CallObject(self->mainFunc, pArgs);
	Py_DECREF(pArgs);
	/*
	Py_DECREF(portsDirP);
	Py_DECREF(workingDirP);
	Py_DECREF(igP);
	*/
	if (pValue != NULL) {
		Py_DECREF(pValue);
	} else {
		PyErr_Print();
		g_critical("Call failed");
		return;
	}
}

static void
LPFilterSessionClass_init (LPFilterSessionClass *selfClass)
{
	LioubovnikFilterSessionClass* lClass=LIOUBOVNIK_FILTER_SESSION_CLASS(selfClass);
	GObjectClass *objectClass = (GObjectClass *)selfClass;

	lpFilterSession_parentClass = g_type_class_peek_parent (selfClass);

	lClass->getInterfacesList=impl_LPFilterSession_getInterfacesList;
	lClass->applyFilter=impl_LPFilterSession_applyFilter;
	objectClass->finalize=impl_LPFilterSession_finalize;
}

GType		LPFilterSession_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (LPFilterSessionClass),
			NULL, NULL,
			(GClassInitFunc) LPFilterSessionClass_init,
			NULL, NULL,
			sizeof (LPFilterSession),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			LIOUBOVNIK_TYPE_FILTER_SESSION,
			"LPFilterSession", &info, 0);
	}

	return type;
}

static gboolean
impl_LPFilterSession_loadModule		(LPFilterSession* self)
{
	gboolean retval=TRUE;
	char* filterPath;

	self->module=NULL;
	self->mainFunc=NULL;
	self->interfacesFunc=NULL;
	filterPath=g_build_filename(
			self->portsDir, 
			((self->filterType==LIOU_FILTER_TYPE__IMPORT)?"Import":"Export"),
			 self->filterName, 
			 "filter.py", 
			 NULL);
	if(!g_file_test(filterPath, G_FILE_TEST_EXISTS|G_FILE_TEST_IS_REGULAR)) {
		/* TODO, filter not present */
		g_critical("Filter %s doesn't exist", filterPath);
		g_free(filterPath);
		return FALSE;
	}
	else {
		char* filterMod;
		char** splitted;
		char* joined;
		PyObject      	*pModule,
			      	*pDict,
			      	*pFunc;


		splitted=g_strsplit(self->filterName, "/", 0);
		joined=g_strjoinv(".", splitted);
		filterMod=g_strconcat(
				((self->filterType==LIOU_FILTER_TYPE__IMPORT)?"Import":"Export"),
				joined,
				".filter",
				NULL);

		g_message("self->filterName:'%s', joined:'%s', filterMod:'%s'", self->filterName, joined, filterMod);
		
		g_free(joined);
		g_strfreev(splitted);
		
		pModule = PyImport_ImportModule(filterMod);
		g_message("1");
		g_message("Is module callable ? : %d", PyCallable_Check(pModule));
		g_message("2");
		if (pModule != NULL) {
			self->module=pModule;
			g_message("2.5");
			g_return_val_if_fail(PyModule_Check(pModule), FALSE);

			g_return_val_if_fail(!PyModule_AddStringConstant(pModule, "workingDir", self->dir), FALSE);
			g_return_val_if_fail(!PyModule_AddStringConstant(pModule, "portsDir", self->portsDir), FALSE);
			g_message("3");
			pDict = PyModule_GetDict(pModule); /* pDict is a borrowed reference */

			g_message("4");
			pFunc = PyDict_GetItemString(pDict, "main"); /* pFun: Borrowed reference */

			if (pFunc && PyCallable_Check(pFunc)) {
				self->mainFunc=pFunc;
			} else {
				if (PyErr_Occurred())
					PyErr_Print();
				g_critical("Cannot find function main");
			}
			pFunc = PyDict_GetItemString(pDict, "interfaces"); /* pFun: Borrowed reference */

			if (pFunc && PyCallable_Check(pFunc)) {
				self->interfacesFunc=pFunc;
			} else {
				if (PyErr_Occurred())
					PyErr_Print();
				g_critical("Cannot find function interfaces");
			}

			Py_DECREF(pModule);
		} else {
			PyErr_Print();
			g_critical("Failed to load \"%s\"\n", filterPath);
			return FALSE;
		}
	}

	g_free(filterPath);
	
	return retval;
}

GObject* 	
LPFilterSession_new			(char* portsDir,
					 char* dir, 
					 LiouFilterType filterType, 
					 char* filterName, 
					 GObject** exception)
{
	LPFilterSession* retval;


	g_message(__PRETTY_FUNCTION__);
	retval=g_object_new(LP_TYPE_FILTER_SESSION, NULL);
	retval->portsDir=g_strdup(portsDir);
	retval->dir=g_strdup(dir);
	retval->filterType=filterType;
	retval->filterName=g_strdup(filterName);

	g_return_val_if_fail(impl_LPFilterSession_loadModule(retval), NULL);
	return (GObject*) retval;
}

